{$INCLUDE ..\cDefines.inc}
{$TYPEINFO ON}
unit cVisualComponents;

interface

uses
  { Delphi }
  Messages,
  Graphics,
  Classes,
  Controls,
  StdCtrls,
  ExtCtrls;



{                                                                              }
{ Revision history:                                                            }
{   13/02/2002  0.01  TfndFramedPanel                                          }
{                                                                              }



{                                                                              }
{ TfndFramedPanel                                                              }
{                                                                              }
type
  TfndFramedPanel = class (TCustomPanel)
  protected
    FFrameColor  : TColor;
    FTitleColor  : TColor;
    FTitleFont   : TFont;
    FTitleHeight : Integer;

    procedure Paint; override;

    procedure SetFrameColor (const FrameColor : TColor);
    procedure SetTitleColor (const TitleColor : TColor);
    procedure SetTitleFont (const TitleFont : TFont);
    procedure SetTitleHeight (const TitleHeight : Integer);

  public
    constructor Create (AOwner : TComponent); override;
    destructor Destroy; override;

  published
    property  Align;
    property  Alignment;
    property  Anchors;
    property  Constraints;
    property  Caption;
    property  Color;
    property  UseDockManager default True;
    property  DockSite;
    property  DragCursor;
    property  DragKind;
    property  DragMode;
    property  Enabled;
    property  FullRepaint;
    property  PopupMenu;
    property  ShowHint;
    property  TabOrder;
    property  TabStop;
    property  Visible;
    property  FrameColor : TColor read FFrameColor write SetFrameColor;
    property  TitleColor : TColor read FTitleColor write SetTitleColor;
    property  TitleFont : TFont read FTitleFont write SetTitleFont;
    property  TitleHeight : Integer read FTitleHeight write SetTitleHeight;
    property  OnCanResize;
    property  OnClick;
    property  OnConstrainedResize;
    property  OnContextPopup;
    property  OnDockDrop;
    property  OnDockOver;
    property  OnDblClick;
    property  OnDragDrop;
    property  OnDragOver;
    property  OnEndDock;
    property  OnEndDrag;
    property  OnEnter;
    property  OnExit;
    property  OnGetSiteInfo;
    property  OnMouseDown;
    property  OnMouseMove;
    property  OnMouseUp;
    property  OnResize;
    property  OnStartDock;
    property  OnStartDrag;
    property  OnUnDock;
  end;



{                                                                              }
{ TfndProgressBar                                                              }
{                                                                              }
type
  TfndProgressBar = class (TGraphicControl)
  protected
    FPosition    : Integer;
    FMaxPosition : Integer;
    FBackColor   : TColor;
    FBarColor    : TColor;
    FRectColor   : TColor;

    procedure Paint; override;

    procedure SetPosition (const Position : Integer);
    procedure SetMaxPosition (const MaxPosition : Integer);
    procedure SetBackColor (const BackColor : TColor);
    procedure SetBarColor (const BarColor : TColor);
    procedure SetRectColor (const RectColor : TColor);

  public
    constructor Create (AOwner : TComponent); override;

  published
    property  Position : Integer read FPosition write SetPosition;
    property  MaxPosition : Integer read FMaxPosition write SetMaxPosition;

    property  BackColor : TColor read FBackColor write SetBackColor;
    property  BarColor : TColor read FBarColor write SetBarColor;
    property  RectColor : TColor read FRectColor write SetRectColor;

    property  Align;
    property  Anchors;
    property  Constraints;
    property  Enabled;
  end;



{                                                                              }
{ TfndDirectoryDialog                                                          }
{                                                                              }
type
  TfndDirectoryDialog = class(TComponent)
  private
    FTitle   : String;
    FDirName : String;
    FDirPath : String;

  public
    function  Execute: Boolean;

  published
    property  Title: String read FTitle write FTitle;
    property  DirName: String read FDirName;
    property  DirPath: String read FDirPath;
  end;



{                                                                              }
{ Component Register                                                           }
{                                                                              }
procedure Register;


implementation

uses
  { Delphi }
  Windows,
  SysUtils,
  Math,
  Types,
  ShlObj,
  Forms;



{                                                                              }
{ TfndFramedPanel                                                              }
{                                                                              }
constructor TfndFramedPanel.Create (AOwner : TComponent);
begin
  inherited Create (AOwner);
  Color := clWhite;
  Alignment := taLeftJustify;
  FFrameColor := clBlack;
  FTitleColor := clSkyBlue;
  FTitleFont := TFont.Create;
  FTitleFont.Name := 'MS Sans Serif';
  FTitleFont.Size := 8;
  FTitleFont.Color := clBlack;
  FTitleFont.Style := [fsBold];
  FTitleHeight := 16;
  ControlStyle := ControlStyle + [csOpaque];
end;

destructor TfndFramedPanel.Destroy;
begin
  FreeAndNil (FTitleFont);
  inherited Destroy;
end;

procedure TfndFramedPanel.Paint;
const
  Alignments : Array [TAlignment] of Longint = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Rect  : TRect;
  I, J  : Integer;
  Flags : LongInt;

begin
  Rect := GetClientRect;
  with Canvas do
    begin
      Brush.Color := FFrameColor;
      FrameRect (Rect);
      With Rect do
        begin
          Inc (Top);
          Inc (Left);
          Dec (Bottom);
          Dec (Right);
        end;
      FrameRect (Rect);

      With Rect do
        begin
          Inc (Top);
          Inc (Left);
          Dec (Right);
          I := Bottom - 1;
          Bottom := Top + FTitleHeight;
        end;
      Brush.Color := FTitleColor;
      FillRect (Rect);

      Font := FTitleFont;
      J := TextHeight ('W');
      With Rect do
        begin
          Top := Top + (FTitleHeight - J) div 2;
          Dec (Right, 8);
        end;
      Flags := DT_EXPANDTABS or DT_VCENTER or Alignments [Alignment];
      Flags := DrawTextBiDiModeFlags(Flags);
      DrawText (Handle, PChar (Caption), -1, Rect, Flags);
      With Rect do
        Inc (Right, 8);

      if FTitleHeight + 4 >= Height then
        exit;
      if FTitleHeight <= 0 then
        exit;

      Pen.Color := FFrameColor;
      Pen.Width := 1;
      MoveTo (Rect.Left, Rect.Bottom);
      LineTo (Rect.Right, Rect.Bottom);

      Brush.Color := Color;
      With Rect do
        begin
          Top := Bottom + 1;
          Bottom := I;
        end;
      FillRect (Rect);
    end;
end;

procedure TfndFramedPanel.SetFrameColor (const FrameColor : TColor);
begin
  FFrameColor := FrameColor;
  Invalidate;
end;

procedure TfndFramedPanel.SetTitleColor (const TitleColor : TColor);
begin
  FTitleColor := TitleColor;
  Invalidate;
end;

procedure TfndFramedPanel.SetTitleFont (const TitleFont : TFont);
begin
  FTitleFont.Assign (TitleFont);
  Invalidate;
end;

procedure TfndFramedPanel.SetTitleHeight (const TitleHeight : Integer);
begin
  FTitleHeight := Min (Max (0, TitleHeight), Height - 4);
  Invalidate;
end;



{                                                                              }
{ TfndProgressBar                                                              }
{                                                                              }
constructor TfndProgressBar.Create (AOwner : TComponent);
begin
  inherited Create (AOwner);
  FBackColor := clBtnFace;
  FBarColor := clSkyBlue;
  FRectColor := clBlack;
  FMaxPosition := 100;
  SetBounds (Left, Top, 96, 12);
end;

procedure TfndProgressBar.Paint;
var Rect, R : TRect;
    W : Integer;
begin
  Rect := GetClientRect;
  With Canvas do
    begin
      Brush.Color := FRectColor;
      FrameRect (Rect);
      With Rect do
        begin
          Inc (Left);
          Dec (Right);
          Inc (Top);
          Dec (Bottom);
        end;
      if FMaxPosition <= 0 then
        W := 0 else
        W := ((Rect.Right - Rect.Left) * Min (FPosition, FMaxPosition)) div FMaxPosition;
      if W > 0 then
        begin
          Brush.Color := FBarColor;
          R := Rect;
          R.Right := R.Left + W;
          FillRect (R);
        end;
      Inc (Rect.Left, W);
      Brush.Color := FBackColor;
      FillRect (Rect);
    end;
end;

procedure TfndProgressBar.SetPosition (const Position : Integer);
begin
  if FPosition = Position then
    exit;
  FPosition := Min (Position, FMaxPosition);
  Invalidate;
end;

procedure TfndProgressBar.SetMaxPosition (const MaxPosition : Integer);
begin
  if FMaxPosition = MaxPosition then
    exit;
  FMaxPosition := Max (1, MaxPosition);
  FPosition := Min (FPosition, FMaxPosition);
  Invalidate;
end;

procedure TfndProgressBar.SetBackColor (const BackColor : TColor);
begin
  if FBackColor = BackColor then
    exit;
  FBackColor := BackColor;
  Invalidate;
end;

procedure TfndProgressBar.SetBarColor (const BarColor : TColor);
begin
  if FBarColor = BarColor then
    exit;
  FBarColor := BarColor;
  Invalidate;
end;

procedure TfndProgressBar.SetRectColor (const RectColor : TColor);
begin
  if FRectColor = RectColor then
    exit;
  FRectColor := RectColor;
  Invalidate;
end;



{                                                                              }
{ TfndDirectoryDialog                                                          }
{                                                                              }
function TfndDirectoryDialog.Execute: Boolean;
var lpItemID    : PItemIDList;
    BrowseInfo  : TBrowseInfo;
    DisplayName : Array[0..MAX_PATH] of Char;
    TempPath    : Array[0..MAX_PATH] of Char;
begin
  FillChar(BrowseInfo, sizeof(TBrowseInfo), #0);
  if Assigned(Owner) and (Owner is TWinControl) then
    BrowseInfo.hwndOwner := TWinControl(Owner).Handle else
    BrowseInfo.hwndOwner := Application.Handle;
  BrowseInfo.pszDisplayName := @DisplayName;
  BrowseInfo.lpszTitle := PChar(FTitle);
  BrowseInfo.ulFlags := BIF_RETURNONLYFSDIRS;
  lpItemID := SHBrowseForFolder(BrowseInfo);
  Result := Assigned(lpItemId);
  if Result then
    begin
      SHGetPathFromIDList(lpItemID, TempPath);
      FDirName := DisplayName;
      FDirPath := TempPath;
      GlobalFreePtr(lpItemID);
    end else
    begin
      FDirName := '';
      FDirPath := '';
    end;
end;



{                                                                              }
{ Component Register                                                           }
{                                                                              }
procedure Register;
begin
  RegisterComponents ('Fundamentals', [TfndFramedPanel, TfndProgressBar,
      TfndDirectoryDialog]);
end;


end.

